/*
 * $Id: keybdev.c,v 1.3 2000/05/28 17:31:36 vojtech Exp $
 *
 *  Copyright (c) 1999-2000 Vojtech Pavlik
 *
 *  Input driver to keyboard driver binding.
 *
 *  Sponsored by SuSE
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Should you need to contact me, the author, you can do so either by
 * e-mail - mail your message to <vojtech@suse.cz>, or by paper mail:
 * Vojtech Pavlik, Ucitelska 1576, Prague 8, 182 00 Czech Republic
 *
 * ChangeLog:
 * 	29-Aug-2003 SHARP Supported USB keybord for Tosa
 * 	11-May-2005 Guylhem - fixed Tosa keyboard support for Bluetooth HIDP
 *
 */

#include <linux/config.h>
#include <linux/kbd_ll.h>
#include <linux/input.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/tty.h>
#include <linux/module.h>
#include <linux/kbd_kern.h>

#if defined(CONFIG_X86) || defined(CONFIG_IA64) || defined(__alpha__) || \
    defined(__mips__) || defined(CONFIG_SPARC64) || defined(CONFIG_SUPERH) || \
    defined(CONFIG_PPC) || defined(__mc68000__) || defined(__hppa__) || \
    defined(__arm__)

static int x86_sysrq_alt = 0;
#ifdef CONFIG_SPARC64
static int sparc_l1_a_state = 0;
extern void batten_down_hatches(void);
#endif

static int keybdev_debug = 0;		/* insmod with keybdev_debug=1 for additional messages */
static int jp_kbd_109 = 1;	/* Yes, .jp is the default. See 51142. */

/* Guylhem: 193 and 194 are null because on a Axim Bluetooth keyboard these are
 * FN green and FN blue
 */

#ifdef CONFIG_ARCH_SHARP_SL

/*
 *  The content of this table is the keycode generated.
 *  If keycode_value > 128, substract 256 to find it!
 */

#ifdef CONFIG_ARCH_PXA_TOSA_NEWKBD_OLDQT
/*
 * Guylhem's keycodes backported from a 2.6 kernel and completed (240->256)
 * 
 * Include the following swaps for OLDQT
 * Z_Bigspace/120                  Z_Record/88
 * Z_Record/88                     Z_Calendar/59
 * Z_Calendar_Held/89              Z_Address/60
 * apostrophe/40                   Z_Home/61
 * control/29                      Z_Menu/62
 * Z_Address/90                    Z_Mail/63
 * Caps_Lock/58                    Z_User/64
 * f/33                            Z_Light/65
 * a/30                            Z_Fn/125
 * 
 * Also had to change, starting from position 96, keycode>256 to
 * keycode=position value, else keypad and edit pad were sharing
 * keycodes
 *
 * Moreover, position 26 and 27 where incorrectly reported because of
 * jp_kbd_109 remapping of zaurus_keycodes in keybdev_init
 */
static unsigned short zaurus_keycodes[256] ={
/*        0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15 */
          0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
/*       16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  31 */
         16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 62,125, 31,
/*       32  33  34  35  36  37  38  39  40  41  42  43  44  45  46  47 */
         32, 65, 34, 35, 36, 37, 38, 39, 61, 41, 42, 43, 44, 45, 46, 47,
/*       48  49  50  51  52  53  54  55  56  57  58  59  60  61  62  63 */
         48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 64, 88, 89, 40, 29, 90,
/*       64  65  66  67  68  69  70  71  72  73  74  75  76  77  78  79 */
         58, 33, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
/*       80  81  82  83  84  85  86  87  88  89  90  91  92  93  94  95 */
         80, 81, 82, 83, 84,118, 86, 87, 99,115, 88,119,121,112,123, 92,
/*       96  97  98  99 100 101 102 103 104 105 106 107 108 109 110 111 */
         96, 97, 98,120,100,101,102,103,104,105,106,107,108,109,110,111,
/*      112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 */
        367,288,302,304,350, 60,334,119,267,126,268,269,125, 30,126,127,
/*      128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 */
        360,261,262,263,268,344,100,101,321,345,373,286,321,102,351,355,
/*      144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 */
        103,104,105,275,287,279,306,106,274,107,294,364,358,363,362,361,
/*      160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 */
        291,108,381,281,290,272,292,305,280, 99,112,257,258,359,113,114,
/*      176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 */
        264,117, 30,374,379,265,266, 93, 94, 95, 85,259,375,260, 63,116,
/*      192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 */
        377,109,111,277,278,282,283,295,317,297,299,300,301,293,303,307,
/*      208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 */
        308,310,313,320,376,296,285,346,314,357,322,323,324,325,276,330,
/*      224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 */
        332,340,365,342,343,315,316,319,356,270,341,368,369,370,371,372,
/*      240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 */
        378,380,383,366,347,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
};
#else // !CONFIG_ARCH_PXA_TOSA_NEWKBD_OLDQT
#ifdef CONFIG_ARCH_PXA_TOSA_NEWKBD
static unsigned short zaurus_keycodes[256] =
	{ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
	 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
	 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
	 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
	 80, 81, 82, 83, 43, 85, 86, 87, 88,115,119,120,121,375,123, 90,
	284,285,309,298,312, 91,327,328,329,331,333,335,336,337,338,339,
	367,294,293,286,350, 92,334,512,116,377,109,111,373,347,348,349,
	360, 93, 94, 95, 98,376,100,101,357,316,354,304,289,102,351,355,
	103,104,105,275,281,272,306,106,274,107,288,364,358,363,362,361,
	291,108,381,290,287,292,279,305,280, 99,112,257,258,113,270,114,
	118,117,125,374,379,259,260,261,262,263,264,265,266,267,268,269,
	271,273,276,277,278,282,283,295,296,297,299,300,301,302,303,307,
	308,310,313,314,315,317,318,319,320,321,322,323,324,325,326,330,
	332,340,341,342,343,344,345,346,356,359,365,368,369,370,371,372 };
#else // !CONFIG_ARCH_PXA_TOSA_NEWKBD + !CONFIG_ARCH_PXA_TOSA_NEWKBD_OLDQT
static unsigned long zaurus_keycodes[256] =
/* OLDKB mode : ported from a SLC. May not work */
       { 0x00, 0x22, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x3a, 0x48, 0x1f, 0x41,
	 0x11, 0x17, 0x05, 0x12, 0x14, 0x19, 0x15, 0x09, 0x0f, 0x10, 0x46, 0x49, 0x1c, 0x68, 0x01, 0x13,
	 0x04, 0x06, 0x07, 0x08, 0x0a, 0x0b, 0x0c, 0x37, 0x38, 0x7a, 0x1b, 0x36, 0x1a, 0x18, 0x03, 0x16,
         0x02, 0x0e, 0x0d, 0x3f, 0x40, 0x0f001b3f, 0x67, 0x3b, 0x1e, 0x5c, 0x68, 0x58, 0x59, 0x5a, 0x28, 0x47,
	 0x00, 0x00, 0x79, 0x1d, 0x00, 0x00, 0x1d, 0x2f, 0x30, 0x31, 0x3a, 0x2c, 0x2d, 0x2e, 0x45, 0x29,
	 0x2a, 0x2b, 0x32, 0x40, 0x49, 0x00, 0x00, 0x00, 0x27, 0x60, 0x5f, 0x00, 0x00, 0x00, 0x00, 0x00,
	 0x1c, 0x68, 0x0f001b3f, 0x28, 0x1e, 0x00, 0x0f006801, 0x24, 0x0f001e24, 0x23, 0x26, 0x0f006805, 0x25, 0x0f001e25, 0x00, 0x51,
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f006803, 0x6d, 0x00, 0x72, 0x51, 0x00, 0x28, 0x28, 0x1d,
	 0x00, 0x00, 0x00, 0x00, 0x45, 0x00, 0x6b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	 0x00, 0x71, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x61,
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x33, 0x79, 0x36, 0x27, 0x22, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
       };

#endif // CONFIG_ARCH_PXA_TOSA_NEWKBD
#endif // CONFIG_ARCH_PXA_TOSA_NEWKBD_OLDQT

#endif // CONFIG_ARCH_SHARP_SL

/*
 * Guylhem: x86_keycodes backported from a 2.6 kernel and completed (240->256)
 */
static unsigned short x86_keycodes[256] ={
/*        0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15 */
          0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
/*       16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  31 */
         16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
/*       32  33  34  35  36  37  38  39  40  41  42  43  44  45  46  47 */
         32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
/*       48  49  50  51  52  53  54  55  56  57  58  59  60  61  62  63 */
         48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
/*       64  65  66  67  68  69  70  71  72  73  74  75  76  77  78  79 */
         64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
/*       80  81  82  83  84  85  86  87  88  89  90  91  92  93  94  95 */
         80, 81, 82, 83, 84,118, 86, 87, 88,115,120,119,121,112,123, 92,
/*       96  97  98  99 100 101 102 103 104 105 106 107 108 109 110 111 */
        284,285,309,298,312, 91,327,328,329,331,333,335,336,337,338,339,
/*      112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 */
        367,288,302,304,350, 89,334,326,267,126,268,269,125,347,348,349,
/*      128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 */
        360,261,262,263,268,376,100,101,321,316,373,286,289,102,351,355,
/*      144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 */
        103,104,105,275,287,279,306,106,274,107,294,364,358,363,362,361,
/*      160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 */
        291,108,381,281,290,272,292,305,280, 99,112,257,258,359,113,114,
/*      176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 */
        264,117,271,374,379,265,266, 93, 94, 95, 85,259,375,260, 90,116,
/*      192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 */
        377,109,111,277,278,282,283,295,296,297,299,300,301,293,303,307,
/*      208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 */
        308,310,313,314,315,317,318,319,320,357,322,323,324,325,276,330,
/*      224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 */
        332,340,365,342,343,344,345,346,356,270,341,368,369,370,371,372,
/*      240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 */
        378,380,383,354,366,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
};


#ifdef CONFIG_MAC_EMUMOUSEBTN
extern int mac_hid_mouse_emulate_buttons(int, int, int);
#endif /* CONFIG_MAC_EMUMOUSEBTN */
#ifdef CONFIG_MAC_ADBKEYCODES
extern int mac_hid_keyboard_sends_linux_keycodes(void);
#else
#define mac_hid_keyboard_sends_linux_keycodes()	0
#endif /* CONFIG_MAC_ADBKEYCODES */
#if defined(CONFIG_MAC_ADBKEYCODES) || defined(CONFIG_ADB_KEYBOARD)
static unsigned char mac_keycodes[256] = {
	  0, 53, 18, 19, 20, 21, 23, 22, 26, 28, 25, 29, 27, 24, 51, 48,
	 12, 13, 14, 15, 17, 16, 32, 34, 31, 35, 33, 30, 36, 54,128,  1,
	  2,  3,  5,  4, 38, 40, 37, 41, 39, 50, 56, 42,  6,  7,  8,  9,
	 11, 45, 46, 43, 47, 44,123, 67, 58, 49, 57,122,120, 99,118, 96,
	 97, 98,100,101,109, 71,107, 89, 91, 92, 78, 86, 87, 88, 69, 83,
	 84, 85, 82, 65, 42,  0, 10,103,111,  0,  0,  0,  0,  0,  0,  0,
	 76,125, 75,105,124,110,115, 62,116, 59, 60,119, 61,121,114,117,
	  0,  0,  0,  0,127, 81,  0,113,  0,  0,  0,  0, 95, 55, 55,  0,
	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	  0,  0,  0,  0,  0, 94,  0, 93,  0,  0,  0,  0,  0,  0,104,102 };
#endif	/* CONFIG_MAC_ADBKEYCODES || CONFIG_ADB_KEYBOARD */
 
static int emulate_raw(unsigned int keycode, int down)
{
#ifdef CONFIG_MAC_EMUMOUSEBTN
	if (mac_hid_mouse_emulate_buttons(1, keycode, down))
		return 0;
#endif /* CONFIG_MAC_EMUMOUSEBTN */
#if defined(CONFIG_MAC_ADBKEYCODES) || defined(CONFIG_ADB_KEYBOARD)
	if (!mac_hid_keyboard_sends_linux_keycodes()) {
		if (keycode > 255 || !mac_keycodes[keycode])
			return -1;
       
		handle_scancode((mac_keycodes[keycode] & 0x7f), down);
		return 0;
	}
#endif	/* CONFIG_MAC_ADBKEYCODES || CONFIG_ADB_KEYBOARD */

#ifdef CONFIG_ARCH_SHARP_SL
	if (keycode > 255 || !zaurus_keycodes[keycode])
		return -1;

//	if (zaurus_keycodes[keycode] & 0x100)
//		handle_scancode(0xe0, 1);

	if (keybdev_debug)
		printk(KERN_WARNING "KEYBDEV reporting keycode=%02x value= %02x state= %s\n", keycode, zaurus_keycodes[keycode], (down ? "Down" : " Up "));

#ifdef CONFIG_ARCH_PXA_TOSA_NEWKBD
	handle_scancode(zaurus_keycodes[keycode] & 0x7f, down);
#else
/* OLDKB mode : ported from SLC. May not work */
       if ( (zaurus_keycodes[keycode] & 0xff000000) && down ) {
               unsigned long   a,b,c,f;

               f = (a = (b = (c = zaurus_keycodes[keycode]) >> 8) >> 8) >> 8;

               a &= 0x7f;
               b &= 0x7f;
               c &= 0x7f;
               f &= 0xff;

               /* Down */
               if ( f & 0x10 ) handle_scancode( a, 1 );
               if ( f & 0x04 ) handle_scancode( b, 1 );
               if ( f & 0x01 ) handle_scancode( c, 1 );

               /* Up */
               if ( f & 0x02 ) handle_scancode( c, 0 );
               if ( f & 0x08 ) handle_scancode( b, 0 );
               if ( f & 0x20 ) handle_scancode( a, 0 );
       }else{
               handle_scancode(zaurus_keycodes[keycode] & 0x7f, down);
       }
#endif // CONFIG_ARCH_PXA_TOSA_NEWKBD
	return 0;
#endif // CONFIG_ARCH_SHARP_SL

	if (keycode > 255 || !x86_keycodes[keycode])
		return -1;

	if (keycode == KEY_PAUSE) {
		handle_scancode(0xe1, 1);
		handle_scancode(0x1d, down);
		handle_scancode(0x45, down);
		return 0;
	} 

	if (keycode == KEY_SYSRQ && x86_sysrq_alt) {
		handle_scancode(0x54, down);

		return 0;
	}

#ifdef CONFIG_SPARC64
	if (keycode == KEY_A && sparc_l1_a_state) {
		sparc_l1_a_state = 0;
		batten_down_hatches();
	}
#endif

	if (x86_keycodes[keycode] & 0x100)
		handle_scancode(0xe0, 1);

	handle_scancode(x86_keycodes[keycode] & 0x7f, down);

	if (keycode == KEY_SYSRQ) {
		handle_scancode(0xe0, 1);
		handle_scancode(0x37, down);
	}

	if (keycode == KEY_LEFTALT || keycode == KEY_RIGHTALT)
		x86_sysrq_alt = down;
#ifdef CONFIG_SPARC64
	if (keycode == KEY_STOP)
		sparc_l1_a_state = down;
#endif

	return 0;
}

#endif /* CONFIG_X86 || CONFIG_IA64 || __alpha__ || __mips__ || CONFIG_PPC */

static struct input_handler keybdev_handler;

static unsigned int ledstate = 0xff;

void keybdev_ledfunc(unsigned int led)
{
	struct input_handle *handle;	

	ledstate = led;

	for (handle = keybdev_handler.handle; handle; handle = handle->hnext) {
		input_event(handle->dev, EV_LED, LED_SCROLLL, !!(led & 0x01));
		input_event(handle->dev, EV_LED, LED_NUML,    !!(led & 0x02));
		input_event(handle->dev, EV_LED, LED_CAPSL,   !!(led & 0x04));
	}
}

void keybdev_event(struct input_handle *handle, unsigned int type, unsigned int code, int down)
{
	if (type != EV_KEY) return;

	if (emulate_raw(code, down))
		printk(KERN_WARNING "keyboard.c: can't emulate rawmode for keycode %d\n", code);

	tasklet_schedule(&keyboard_tasklet);
}

static struct input_handle *keybdev_connect(struct input_handler *handler, struct input_dev *dev)
{
	struct input_handle *handle;
	int i;

	if (!test_bit(EV_KEY, dev->evbit))
		return NULL;

	for (i = KEY_RESERVED; i < BTN_MISC; i++)
		if (test_bit(i, dev->keybit)) break;

	if (i == BTN_MISC)
 		return NULL;

	if (!(handle = kmalloc(sizeof(struct input_handle), GFP_KERNEL)))
		return NULL;
	memset(handle, 0, sizeof(struct input_handle));

	handle->dev = dev;
	handle->handler = handler;

	input_open_device(handle);

	if (keybdev_debug)
		printk(KERN_INFO "KEYBDEV adding keyboard: input%d\n", dev->number);

	if (ledstate != 0xff) {
		input_event(dev, EV_LED, LED_SCROLLL, !!(ledstate & 0x01));
		input_event(dev, EV_LED, LED_NUML,    !!(ledstate & 0x02));
		input_event(dev, EV_LED, LED_CAPSL,   !!(ledstate & 0x04));
	}

	return handle;
}

static void keybdev_disconnect(struct input_handle *handle)
{
	if (keybdev_debug)
		printk(KERN_INFO "KEYBDEV removing keyboard: input%d\n", handle->dev->number);

	input_close_device(handle);
	kfree(handle);
}
	
static struct input_handler keybdev_handler = {
	event:		keybdev_event,
	connect:	keybdev_connect,
	disconnect:	keybdev_disconnect,
};

static int __init keybdev_init(void)
{
	input_register_handler(&keybdev_handler);
	kbd_ledfunc = keybdev_ledfunc;

	if (jp_kbd_109) {
#if !defined (CONFIG_ARCH_PXA_TOSA_NEWKBD) && !defined(CONFIG_ARCH_SHARP_SL)
		x86_keycodes[0xb5] = 0x73;	/* backslash, underscore */
		x86_keycodes[0xb6] = 0x70;
		x86_keycodes[0xb7] = 0x7d;	/* Yen, pipe */
		x86_keycodes[0xb8] = 0x79;
		x86_keycodes[0xb9] = 0x7b;
#else
		/* Guylhem: does no good to Tosa keyboard, since we
		 * already do the whole remapping by hand
		 */
//               zaurus_keycodes[0xb5] = 0x33;      /* ro, backslash, underscore */
//               zaurus_keycodes[0xb6] = 0x79;      /* katakana-hiragana */
//               zaurus_keycodes[0xb7] = 0x36;      /* Yen, pipe */
//               zaurus_keycodes[0xb8] = 0x27;      /* Henkan */
//               zaurus_keycodes[0xb9] = 0x22;      /* Mu-Henkan */
//               zaurus_keycodes[0x1a] = 0x55;      /* Henkan */
//               zaurus_keycodes[0x1b] = 0x46;      /* Mu-Henkan */
#endif
	}

	return 0;
}

static void __exit keybdev_exit(void)
{
	kbd_ledfunc = NULL;
	input_unregister_handler(&keybdev_handler);
}

module_init(keybdev_init);
module_exit(keybdev_exit);

MODULE_AUTHOR("Vojtech Pavlik <vojtech@suse.cz>");
MODULE_DESCRIPTION("Input driver to keyboard driver binding");
MODULE_PARM(jp_kbd_109, "i");
MODULE_PARM(keybdev_debug, "i");
MODULE_LICENSE("GPL");
