/*
 * TI TSC2102 Common Code
 *
 * Copyright 2005 Openedhand Ltd.
 *
 * Author: Richard Purdie <richard@o-hand.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/delay.h>
#include <linux/device.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/soc/tsc2101.h> 
#include "tsc2101.h"



//struct tsc2101_dev_params = {
//	.sendcmd = ,
//	.write   =,
//};
extern int hx2750_tsc2101_send(int write, int command, int *values, int numval);

static int tsc2101_regread(int regnum)
{
	int reg;
	hx2750_tsc2101_send(TSC2101_READ, regnum, &reg, 1);
	return reg;
}

static void tsc2101_regwrite(int regnum, int value)
{
	int reg=value;
	hx2750_tsc2101_send(TSC2101_WRITE, regnum, &reg, 1);
	return;
}

static int tsc2101_ts_penup(void)
{
	return !( tsc2101_regread(TSC2101_REG_STATUS) & TSC2101_STATUS_DAVAIL);
}

static void tsc2101_ts_read(struct tsc2101_ts_event *ts_data)
{
	int z1,z2;
	u32 values[4];

	/* Read X, Y, Z1 and Z2 */
	hx2750_tsc2101_send(TSC2101_READ, TSC2101_REG_X, &values[0], 4);

	ts_data->x=values[0];
	ts_data->y=values[1];
	z1=values[2];
	z2=values[3];

	/* Calculate Pressure */
	if ((z1 != 0) && (ts_data->x!=0) && (ts_data->y!=0)) 
		ts_data->p = ((ts_data->x * (z2 -z1) / z1));
	else
		ts_data->p=0;
}

#define TSC2101_DATA_AVAILABLE         0x4000
#define TSC2101_BUFFERMODE_DISABLE     0x0
#define TSC2101_REF_POWERUP            0x16
#define TSC2101_ENABLE_TOUCHDETECT     0x08
#define TSC2101_PRG_DELAY              0x0900
#define TSC2101_ADC_CONTROL            0x8874
#define TSC2101_ADC_POWERDOWN          0x4000

static void tsc2101_ts_enable(void)
{
	//omap_tsc2101_enable();
	
	/* PINTDAV is data available only */
	tsc2101_regwrite(TSC2101_REG_STATUS, TSC2101_DATA_AVAILABLE);

	/* disable buffer mode */
	tsc2101_regwrite(TSC2101_REG_BUFMODE, TSC2101_BUFFERMODE_DISABLE);

	/* use internal reference, 100 usec power-up delay,
	 *	  * power down between conversions, 1.25V internal reference */
	tsc2101_regwrite(TSC2101_REG_REF, TSC2101_REF_POWERUP);
			   
	/* enable touch detection, 84usec precharge time, 32 usec sense time */
	tsc2101_regwrite(TSC2101_REG_CONFIG, TSC2101_ENABLE_TOUCHDETECT);
			   
	/* 3 msec conversion delays  */
	tsc2101_regwrite(TSC2101_REG_DELAY, TSC2101_PRG_DELAY);
	
	/*
	 * TSC2101-controlled conversions
	 * 12-bit samples
	 * continuous X,Y,Z1,Z2 scan mode
	 * average (mean) 4 samples per coordinate
	 * 1 MHz internal conversion clock
	 * 500 usec panel voltage stabilization delay
	 */
	tsc2101_regwrite(TSC2101_REG_ADC, TSC2101_ADC_CONTROL);
//0x8874

	return;

}

static void tsc2101_ts_disable(void)
{
	//printk("tsc2101_ts_disable\n");
	/* stop conversions and power down */
	tsc2101_regwrite(TSC2101_REG_ADC, TSC2101_ADC_POWERDOWN);

	//omap_tsc2101_disable();
}




struct ts_device hx_ts = {
//	.probe 		= tsc2101_ts_probe,
	.read 		= tsc2101_ts_read,
	.enable 	= tsc2101_ts_enable,
	.disable 	= tsc2101_ts_disable,
	.remove 	= __exit_p(tsc2101_ts_remove),
	.penup 		= tsc2101_ts_penup,
};

